XIncludeFile "OWire_Module.pbi" ;   .
XIncludeFile "USB HID.pbi"
XIncludeFile ".pbf"         ;   .

UseModule OWire ;       "OWire_Module.pbi".

Enumeration OW_ErrCode
  #OW_Err_CRC ;  CRC.
EndEnumeration

#USB_PID    = $2010  ;  USB .
#USB_VID    = $0207

#PrTemp_Def = 90.0 ;    .

Structure DS ;    .
  ID.OW_SN    ;  .
  Temp.f      ;   .
  ErrCode.a   ;  .
  PrTemp.f    ;  
EndStructure

Global NewList DS_List.DS() ;  .
Global W_DeviceHandle        ;    USB.

Procedure.s GetErrString(Err) ;   .
  Protected String.s
  
  Select Err
    Case #OW_Err_OK       : String = " "
    Case #OW_Err_Port     : String = "  COM "
    Case #OW_Err_NoDevice : String = "   '1-Wire' "
    Case #OW_Err_IO       : String = "  "
    Case #OW_Err_BaudRate : String = "   "
    Case #OW_Err_Search   : String = "   '1-Wire' "
    Case #OW_Err_BadParam : String = "   "
    Case #OW_Err_NoData   : String = " "
    Case #OW_Err_CRC      : String = " CRC"
  EndSelect
  
  ProcedureReturn String
EndProcedure

Procedure.s SN_Text(*x.OW_SN)
  Protected String.s, i
  
  String = RSet(Hex(*x\Type), 2, "0")+"-"
  
  For i=5 To 0 Step -1
    String + RSet(Hex(*x\Number[i]), 2, "0")
  Next i
  
  String + "-"+RSet(Hex(*x\CRC), 2, "0")
  
  ProcedureReturn String
EndProcedure

Procedure Find() ;  
  ID.OW_SN ;    .
  
  If OW_PortID()=0
    MessageRequester("", "  COM ")
    ProcedureReturn ;   .
  EndIf
  
  ClearList(DS_List())          ;  .
  ClearGadgetItems(#ListIcon_0) ;    .
  
  x=0 ;   .
  
  OW_SearchROM_Reset()
  
  While OW_SearchROM_Enum(ID)=#True
    If (ID\Type = $28 Or ID\Type = $10) And AddElement(DS_List())
      DS_List()\ID = ID
      DS_List()\ErrCode = 255
      DS_List()\PrTemp = #PrTemp_Def ;    .
      AddGadgetItem(#ListIcon_0, x, SN_Text(DS_List()\ID)+#LF$+""+#LF$+StrF(#PrTemp_Def, 1)+#LF$+Str(x+1)+#LF$+"")
      x+1 ;  1    .
    EndIf
  Wend
  
EndProcedure

Procedure FindDevice() ;     USB.
  Static Old_Test=-1
  
  x=HID::TestDevice(#USB_PID, #USB_VID)
  If x<>Old_Test
    Old_Test=x
    If x ;  .
      HID::CloseDevice(W_DeviceHandle)
      W_DeviceHandle=HID::OpenDevice(#USB_PID, #USB_VID)
      SetGadgetText(#Text_1," ")
    Else ;  .
      HID::CloseDevice(W_DeviceHandle)
      W_DeviceHandle=0
      SetGadgetText(#Text_1," ")
    EndIf
  EndIf
  
EndProcedure

Procedure TempTest() ;      .
  USB_Out.a=0
  x=0
  
  If W_DeviceHandle ;    USB.
    ForEach DS_List()
      
      If DS_List()\ErrCode = 0 ;  .
        If DS_List()\Temp >= DS_List()\PrTemp ;   .
          USB_Out = USB_Out | (1<<x)
          SetGadgetItemText(#ListIcon_0, x, "", 4)
        Else
          SetGadgetItemText(#ListIcon_0, x, "", 4)
        EndIf
      EndIf
      
      x+1
      If x>=8
        Break ;  8    .
      EndIf
      
    Next
    
    Dim USB.a(2) ;  ,   USB.
    
    USB(0)=0
    USB(1)=USB_Out
    
    HID::WriteDevice(W_DeviceHandle, @USB(), 2) ;  2   USB.
  EndIf
  
EndProcedure

Procedure Timer() ;    
  RAM.OW_RAM
  x=0 ;   .
  
  FindDevice() ;     USB.
  
  ForEach DS_List()
    
    If DS_List()\ErrCode=0 ;   .
      If OW_SelectROM(DS_List()\ID) ;  .
        If OW_ReadRAM(RAM)          ;    .
          If OW_GetCRC(RAM, 8)=RAM\CRC
            
            DS_List()\Temp = PeekW(RAM) ;  ( ).
            
            ;  .
            If DS_List()\ID\Type = $28 ; DS18B20
              DS_List()\Temp / 16
            Else                       ; DS1820  DS18S20
              DS_List()\Temp / 2
            EndIf
            
            DS_List()\ErrCode = 0 ;   
          Else
            DS_List()\ErrCode = #OW_Err_CRC
          EndIf
        Else
          DS_List()\ErrCode = OW_GetLastError()
        EndIf
      Else
        DS_List()\ErrCode = OW_GetLastError()
      EndIf
    EndIf
    
    If DS_List()\ErrCode = 0
      SetGadgetItemText(#ListIcon_0, x, StrF(DS_List()\Temp, 1), 1)
    Else
      SetGadgetItemText(#ListIcon_0, x, GetErrString(DS_List()\ErrCode), 1)
    EndIf
    
    x+1 ;  1    .
  Next
  
  TempTest() ;      .
  
  ;        .
  If OW_SkipROM()=0 Or OW_ConvertT()=0
    ErrCode = OW_GetLastError()
  Else
    ErrCode = 0 ;  .
  EndIf
  
  ForEach DS_List()
    DS_List()\ErrCode = ErrCode
  Next
    
EndProcedure

HID::HID_Init() ;  USB.

OpenWindow_0()

AddWindowTimer(#Window_0, 2, 1000) ;     1 .

Repeat
  Event = WaitWindowEvent()
  
  If Event = #PB_Event_Gadget
    
    Select EventGadget()
      Case #Combo_0    ;    COM .
        OW_OpenPort(GetGadgetText(#Combo_0)) ;  .
      Case  #Button_0  ;    .
        Find()         ;    
      Case #ListIcon_0 ;      .
        If EventType() = #PB_EventType_LeftDoubleClick ;    .
          x = GetGadgetState(#ListIcon_0) ;      .
          If x >= 0 And x < ListSize(DS_List())
            If SelectElement(DS_List(), x) ;   .
              PrT.s=InputRequester("", " ", StrF(DS_List()\PrTemp, 1))
              If PrT<>"" ;   .
                DS_List()\PrTemp = ValF(PrT) ;     
                SetGadgetItemText(#ListIcon_0, x, StrF(DS_List()\PrTemp, 1), 2)
              EndIf
            EndIf
          EndIf
        EndIf
    EndSelect
    
  ElseIf Event = #PB_Event_Timer ;  .
    If EventTimer() = 2 ;     2.
      Timer()           ;    
    EndIf
  EndIf
  
Until Event = #PB_Event_CloseWindow

HID::HID_End()
; IDE Options = PureBasic 5.24 LTS (Windows - x86)
; Folding = --
; EnableAsm
; EnableXP
; Executable = TestTemp 1.2.exe
; HideErrorLog
; EnablePurifier
; EnableExeConstant